
// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2008 - INRIA - Michael Baudin
// Copyright (C) 2009 - Digiteo - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

#include <limits.h>

#include "api_scilab.h"
#include "stack-c.h" 
#include "Scierror.h"
#include "localization.h"

#include "gw_distfungrand_support.h" 
#include "gwsupport.h" 

//   flag=0 if the size of the parameter a is wrong.
static int GRANDGW_SIZEERROR = 0;
//   flag=1 if size(a)=[1 1], 
static int GRANDGW_SIZESCALAR = 1;
//   flag=2 if size(a)=[m n], 
static int GRANDGW_SIZEMN = 2;


int distfun_CheckExpansionCaseA ( char * fname , int m, int n, int ma , int na );
int distfun_CheckExpansionCaseAB ( char * fname , int m, int n, int ma , int na , int mb , int nb );
int distfun_CheckExpansionCaseABC ( char * fname , int m, int n, int ma , int na , int mb , int nb , int mc , int nc );
int distfun_WhatParameterSize ( char * fname , int m, int n, int ma , int na );

int distfun_GetMN ( char * fname , int * nrows , int * ncols )
{
	int readFlag;
	double doubleNrows;
	double doubleNcols;

	// Get Arg #1 = M
	readFlag = gwsupport_GetOneIntegerArgument ( fname , 1, nrows );
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return GWSUPPORT_ERROR;
	}
	doubleNrows = (double) (*nrows);
	readFlag = gwsupport_CheckDoubleGreaterOrEqual (fname, 1, doubleNrows, 0.0);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return GWSUPPORT_ERROR;
	}
	// Get Arg #2 = N
	readFlag = gwsupport_GetOneIntegerArgument ( fname , 2, ncols );
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return GWSUPPORT_ERROR;
	}
	doubleNcols = (double) (*ncols);
	readFlag = gwsupport_CheckDoubleGreaterOrEqual (fname, 2, doubleNcols, 0.0);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return GWSUPPORT_ERROR;
	}
	return GWSUPPORT_OK;
}

// 
// distfun_WhatParameterSize --
//   Returns flag=GRANDGW_SIZESCALAR if size(a)=[1 1], 
//   returns flag=GRANDGW_SIZEMN if size(a)=[m n], 
//   returns flag=GRANDGW_SIZEERROR if the size of the parameter a is wrong.
// Arguments
//   fname (input) : the name of the Scilab function generating this error
//   m (input) : the required number of rows
//   n (input) : the required number of columns
//   ma (input) : the number of rows in a
//   na (input) : the number of columns in a
// Description
// Use this function to expand parameters.
//
int distfun_WhatParameterSize ( char * fname , int m, int n, int ma , int na )
{
	int flag;
	if ( ma==1 && na==1 ) 
	{ 
		flag = GRANDGW_SIZESCALAR;
	} 
	else if ( ma==m && na==n ) 
	{ 
		flag = GRANDGW_SIZEMN;
	} 
	else
	{
		flag = GRANDGW_SIZEERROR;
	}
	return flag;
}

// 
// distfun_CheckExpansionCaseA --
//   Returns expcase=GRANDGW_SIZESCALAR if (a is 1-by-1 matrix),
//   or expcase=GRANDGW_SIZEMN if (size(a) == m-by-n)
//   or expcase=GRANDGW_SIZEERROR (error) if no case match.
// Arguments
//   fname (input) : the name of the Scilab function generating this error
//   m (input) : the number of rows required by the user
//   n (input) : the number of columns required by the user
//   ma (input) : the number of rows in a
//   na (input) : the number of columns in a
// Description
// Use this function to expand distfun_grandXXX(m,n,a) functions 
// to the case where a is non-scalar.
//
int distfun_CheckExpansionCaseA ( char * fname , int m, int n, int ma , int na )
{
	int expcase;
	int flag_a;
	flag_a = distfun_WhatParameterSize ( fname , m, n, ma , na );

	// Check the cases
	if ( flag_a==GRANDGW_SIZESCALAR ) 
	{ 
		expcase = GRANDGW_SIZESCALAR;
	} 
	else if ( flag_a == GRANDGW_SIZEMN )
	{
		expcase = GRANDGW_SIZEMN;
	}
	else
	{
		// a is wrong
		Scierror(999,_("%s: Wrong size for input argument #%d: %d-by-%d matrix expected.\n"),
			fname, 3 , m, n );
		expcase = GRANDGW_SIZEERROR;
	}
	return expcase;
}

// 
// distfun_CheckExpansionCaseAB --
//   Returns expcase=GRANDGW_SIZESCALAR if (a and b are 1-by-1 matrices),
//   or expcase=GRANDGW_SIZEMN if (size(a) == m-by-n and size(b)== m-by-n)
//   or expcase=GRANDGW_SIZEERROR (error) if no case match.
// Arguments
//   fname (input) : the name of the Scilab function generating this error
//   ma (input) : the number of rows in a
//   na (input) : the number of columns in a
//   mb (input) : the number of rows in b
//   nb (input) : the number of columns in b
// Description
// Use this function to expand distfun_grandXXX(m,n,a,b) functions 
// to the case where a and b are non-scalars.
//
int distfun_CheckExpansionCaseAB ( char * fname , int m, int n, int ma , int na , int mb , int nb )
{
	int expcase;
	int flag_a;
	int flag_b;
	flag_a = distfun_WhatParameterSize ( fname, m, n, ma, na );
	flag_b = distfun_WhatParameterSize ( fname, m, n, mb, nb );

	// Check the cases
	if ( flag_a==GRANDGW_SIZESCALAR && flag_b==GRANDGW_SIZESCALAR )
	{ 
		expcase = GRANDGW_SIZESCALAR;
	} 
	else if ( flag_a==GRANDGW_SIZEMN && flag_b==GRANDGW_SIZEMN ) 
	{
		expcase = GRANDGW_SIZEMN;
	}
	else
	{
		if ( flag_a==GRANDGW_SIZEERROR )
		{
			// a is wrong
			Scierror(999,_("%s: Wrong size for input argument #%d: %d-by-%d matrix expected.\n"),
				fname, 3 , m, n );
			expcase = GRANDGW_SIZEERROR;
		}
		else
		{
			// b is wrong
			Scierror(999,_("%s: Wrong size for input argument #%d: %d-by-%d matrix expected.\n"),
				fname, 4 , m, n );
			expcase = GRANDGW_SIZEERROR;
		}
	}
	return expcase;
}
// 
// distfun_CheckExpansionCaseABC --
//   Returns expcase=GRANDGW_SIZESCALAR if (a and b and c are 1-by-1 matrices),
//   or expcase=GRANDGW_SIZEMN if (size(a) == m-by-n and size(b)== m-by-n and size(c)== m-by-n)
//   or expcase=GRANDGW_SIZEERROR (error) if no case match.
// Arguments
//   fname (input) : the name of the Scilab function generating this error
//   ma (input) : the number of rows in a
//   na (input) : the number of columns in a
//   mb (input) : the number of rows in b
//   nb (input) : the number of columns in b
//   mc (input) : the number of rows in c
//   nc (input) : the number of columns in c
// Description
// Use this function to expand distfun_grandXXX(m,n,a,b,c) functions 
// to the case where a and b are non-scalars.
//
int distfun_CheckExpansionCaseABC ( char * fname , int m, int n, int ma , int na , int mb , int nb , int mc , int nc )
{
	int expcase;
	int flag_a;
	int flag_b;
	int flag_c;
	flag_a = distfun_WhatParameterSize ( fname, m, n, ma, na );
	flag_b = distfun_WhatParameterSize ( fname, m, n, mb, nb );
	flag_c = distfun_WhatParameterSize ( fname, m, n, mc, nc );

	// Check the cases
	if ( flag_a==GRANDGW_SIZESCALAR && flag_b==GRANDGW_SIZESCALAR && flag_c==GRANDGW_SIZESCALAR ) 
	{ 
		expcase = GRANDGW_SIZESCALAR;
	} 
	else if (flag_a==GRANDGW_SIZEMN && flag_b==GRANDGW_SIZEMN && flag_c==GRANDGW_SIZEMN )
	{
		expcase = GRANDGW_SIZEMN;
	}
	else
	{
		if ( flag_a==GRANDGW_SIZEERROR )
		{
			// a is wrong
			Scierror(999,_("%s: Wrong size for input argument #%d: %d-by-%d matrix expected.\n"),
				fname, 3 , m, n );
			expcase = GRANDGW_SIZEERROR;
		}
		else if ( flag_b==GRANDGW_SIZEERROR )
		{
			// b is wrong
			Scierror(999,_("%s: Wrong size for input argument #%d: %d-by-%d matrix expected.\n"),
				fname, 4 , m, n );
			expcase = GRANDGW_SIZEERROR;
		}
		else
		{
			// c is wrong
			Scierror(999,_("%s: Wrong size for input argument #%d: %d-by-%d matrix expected.\n"),
				fname, 5 , m, n );
			expcase = GRANDGW_SIZEERROR;
		}
	}
	return expcase;
}



int distfun_computeRandgenAB (char *fname, 
	int ma, int na, double * pa, 
	int mb, int nb, double * pb, 
	int rRows, int rCols,  
	double (*randgen)(double pa , double pb) )
{
	int i;
	int expcase;
	double * pr = NULL;
	double randvalue;
	int readFlag;

	// See the expansion case
	expcase = distfun_CheckExpansionCaseAB ( fname , rRows, rCols, ma , na , mb , nb );
	if ( expcase == GRANDGW_SIZEERROR)
	{ 
		return GWSUPPORT_ERROR;
	}
	// Create Lhs
	readFlag = gwsupport_AllocateLhsMatrixOfDoubles ( 1, rRows, rCols, &pr);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return GWSUPPORT_ERROR;
	}
	// Now compute the result
	if ( expcase == GRANDGW_SIZESCALAR)
	{
		// a and b are scalars
		for ( i=0 ; i < rRows*rCols ; i++)
		{
			randvalue = (*randgen)(*pa,*pb);
			*(pr+i) = randvalue;
		}
	}
	else
	{
		// a and b are matrices
		for ( i=0 ; i < rRows*rCols ; i++)
		{
			randvalue = (*randgen)(pa[i],pb[i]);
			*(pr+i) = randvalue;
		}
	}
	return GWSUPPORT_OK;
}

int distfun_computeRandgenA (char *fname, 
	int ma, int na, double * pa, 
	int rRows, int rCols, 
	double (*randgen)(double pa) )
{
	int i;
	int expcase;
	double * pr = NULL;
	double randvalue;
	int readFlag;

	// See the expansion case
	expcase = distfun_CheckExpansionCaseA ( fname , rRows, rCols, ma , na );
	if ( expcase == GRANDGW_SIZEERROR)
	{ 
		return GWSUPPORT_ERROR;
	}
	// Create Lhs
	readFlag = gwsupport_AllocateLhsMatrixOfDoubles ( 1, rRows, rCols, &pr);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return GWSUPPORT_ERROR;
	}
	// Now compute the result
	if ( expcase == GRANDGW_SIZESCALAR)
	{
		// a is scalar
		for ( i=0 ; i < rRows*rCols ; i++)
		{
			randvalue = (*randgen)(*pa);
			*(pr+i) = randvalue;
		}
	}
	else
	{
		// a is vector
		for ( i=0 ; i < rRows*rCols ; i++)
		{
			randvalue = (*randgen)(pa[i]);
			*(pr+i) = randvalue;
		}
	}
	return GWSUPPORT_OK;
}


int distfun_computeRandgenABC (char *fname, 
	int ma, int na, double * pa, 
	int mb, int nb, double * pb, 
	int mc, int nc, double * pc, 
	int rRows, int rCols,  
	double (*randgen)(double pa , double pb, double pc) )
{
	int i;
	int expcase;
	double * pr = NULL;
	double randvalue;
	int readFlag;

	// See the expansion case
	expcase = distfun_CheckExpansionCaseABC ( fname , rRows, rCols, ma , na , mb , nb , mc , nc );
	if ( expcase == GRANDGW_SIZEERROR)
	{ 
		return GWSUPPORT_ERROR;
	}
	// Create Lhs
	readFlag = gwsupport_AllocateLhsMatrixOfDoubles ( 1, rRows, rCols, &pr);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return GWSUPPORT_ERROR;
	}
	// Now compute the result
	if ( expcase == GRANDGW_SIZESCALAR)
	{
		// a and b and c are scalars
		for ( i=0 ; i < rRows*rCols ; i++)
		{
			randvalue = (*randgen)(*pa,*pb,*pc);
			*(pr+i) = randvalue;
		}
	}
	else
	{
		// a and b and c are matrices
		for ( i=0 ; i < rRows*rCols ; i++)
		{
			randvalue = (*randgen)(pa[i],pb[i],pc[i]);
			*(pr+i) = randvalue;
		}
	}
	return GWSUPPORT_OK;
}
