/*
*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 1994 - Barry W. Brown, James Lovato, Kathy Russell (DCDFLIB)
*
*/

/* cdfchi.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include <math.h>

#include "cdflib.h"
#include "cdflib_private.h" 


/* ********************************************************************** */

// Chi-Square Distribution

/*                              Method */

/*     Formula    26.4.19   of Abramowitz  and     Stegun, Handbook  of */
/*     Mathematical Functions   (1966) is used   to reduce the chisqure */
/*     distribution to the incomplete distribution. */

/*     Computation of other parameters involve a seach for a value that */
/*     produces  the desired  value  of P.   The search relies  on  the */
/*     monotinicity of P with the other parameter. */

/* ********************************************************************** */

/* Table of constant values */

static int c__1 = 1;
static double c_b25 = 0.;
static double c_b27 = .5;
static double c_b29 = 5.;

/* Subroutine */ int cdfchi_(int which, double *p, double *q, 
	double *x, double k)
{
	int status;

	/* Local variables */
	static double fx, pq;
	static long int qhi;
	static double cum, ccum, porq;
	static long int qleft;
	static long int qporq;

	double large =cdflib_doubleLarge();
	double rtol = cdflib_doubleEps();
	double infinite = cdflib_infinite();
	double atol = cdflib_doubleTiny();


	/*     Check arguments */

	status=cdflib_checkrangeint("cdfchi", which, "which", 1, 2);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/* P, Q, P + Q */
	if (which==2)
	{
		status = cdflib_checkpq("cdfchi", *p, "p", *q, "q");
		if (status!=CDFLIB_OK)
		{
			return status;
		}
	}

	// Check X
	if (which==1)
	{
		status = cdflib_chiCheckX("cdfchi",*x);
		if (status!=CDFLIB_OK)
		{
			return status;
		}
	}

	// Check k
	status = cdflib_chiCheckParams("cdfchi",k);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/*     Select the minimum of P or Q */
	if (which == 2) 
	{
		if (*p <= *q) 
		{
			porq = *p;
		}
		else
		{
			porq = *q;
		}
	}

	/*     Calculate ANSWERS */

	if (which==1) {

		/*     Calculating P and Q */
		if (*x==infinite)
		{
			*p=1.;
			*q=0.;
			status = CDFLIB_OK;
			return status;
		}
		if (cdflib_isnan(*x) || cdflib_isnan(k))
		{
			*p = *x+k;
			*q = *x+k;
			status = CDFLIB_OK;
			return status;
		}
		status = CDFLIB_OK;
		cdflib_cumchi(*x, k, p, q);
		if (*p > 1.5 || *q > 1.5) {
			status = 10;
			return status;
		}
	} else if (which==2) {

		/*     Calculating X */
		if ( *p == 0. ) 
		{
			*x = 0.;
			status = CDFLIB_OK;
			return status;
		}
		if ( *q == 0. ) 
		{
			*x = infinite;
			status = CDFLIB_OK;
			return status;
		}
		if (cdflib_isnan(*p) || cdflib_isnan(*q) || cdflib_isnan(k))
		{
			*x = *p+*q+k;
			status = CDFLIB_OK;
			return status;
		}
		*x = 5.;
		cdflib_dstinv(&c_b25, &large, &c_b27, &c_b27, &c_b29, &atol, &rtol);
		status = 0;
		cdflib_dinvr(&status, x, &fx, &qleft, &qhi);
		while (status == 1)
		{
			cdflib_cumchi(*x, k, &cum, &ccum);
			fx = cdflib_computefx(*p, *q, cum, ccum);
			if (fx + porq > 1.5) 
			{
				status = 10;
				return status;
			}
			cdflib_dinvr(&status, x, &fx, &qleft, &qhi);
		}
		if (status == 0) 
		{
			status = CDFLIB_OK;
		}
		else
		{
			if (qleft) 
			{
				cdflib_unabletoinvert("cdfchi", 0., "x");
				status = CDFLIB_ERROR;
			}
			else
			{
				cdflib_unabletoinvert("cdfchi", large, "x");
				status = CDFLIB_ERROR;
			}
		}
	}
	return status;
}

int cdflib_chiCheckParams(char * fname, double k)
{
	int status;
	/*     K > 0 */
	// k can be non integer
	status = cdflib_checkgreaterthan(fname, k, "k", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}
int cdflib_chiCheckX(char * fname, double x)
{
	int status;
	/*     X >=0 */
	status = cdflib_checkgreqthan(fname, x, "x", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}
int cdflib_chi2cdf(double x, double k, int lowertail, double *p)
{
	int status;
	double q;
	double infinite = cdflib_infinite();
	status = cdflib_chiCheckX("cdflib_chi2cdf",x);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Check k
	status = cdflib_chiCheckParams("cdflib_chi2cdf",k);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	if (x==infinite)
	{
		if (lowertail==CDFLIB_LOWERTAIL)
		{
			*p=1.;
		}
		else
		{
			*p=0.;
		}
		status = CDFLIB_OK;
		return status;
	}
	if (cdflib_isnan(x) || cdflib_isnan(k))
	{
		*p = x+k;
		status = CDFLIB_OK;
		return status;
	}
	cdflib_cumchi(x, k, p, &q);
	if (*p > 1.5 || q > 1.5) 
	{
		status = CDFLIB_ERROR;
		return status;
	}
	if (lowertail==CDFLIB_UPPERTAIL)
	{
		*p=q;
	}
	status = CDFLIB_OK;
	return status;
}
int cdflib_chi2inv(double p, double k, int lowertail, double *x)
{
	int status;
	/* Local variables */
	static double fx, pq;
	static long int qhi;
	static double cum, ccum, porq;
	static long int qleft;
	static long int qporq;

	double large =cdflib_doubleLarge();
	double rtol = cdflib_doubleEps();
	double infinite = cdflib_infinite();
	double atol = cdflib_doubleTiny();
	double q;

	// P in [0,1]
	status=cdflib_checkp("cdflib_chi2inv", p, "p");
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Check k
	status = cdflib_chiCheckParams("cdflib_chi2inv",k);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Compute the couple (p,q) from (p,lowertail)
	if (lowertail==CDFLIB_LOWERTAIL)
	{
		q=1-p;
	}
	else
	{
		q=p;
		p=1-q;
	}

	if (p <= q) 
	{
		porq = p;
	}
	else
	{
		porq = q;
	}

	if ( p == 0. ) 
	{
		*x = 0.;
		status = CDFLIB_OK;
		return status;
	}
	if ( q == 0. ) 
	{
		*x = infinite;
		status = CDFLIB_OK;
		return status;
	}
	if (cdflib_isnan(p) || cdflib_isnan(k))
	{
		*x = p+k;
		status = CDFLIB_OK;
		return status;
	}
	*x = 5.;
	cdflib_dstinv(&c_b25, &large, &c_b27, &c_b27, &c_b29, &atol, &rtol);
	status = 0;
	cdflib_dinvr(&status, x, &fx, &qleft, &qhi);
	while (status == 1)
	{
		cdflib_cumchi(*x, k, &cum, &ccum);
		fx = cdflib_computefx(p, q, cum, ccum);
		if (fx + porq > 1.5) 
		{
			status = CDFLIB_ERROR;
			return status;
		}
		cdflib_dinvr(&status, x, &fx, &qleft, &qhi);
	}
	if (status == 0) 
	{
		status = CDFLIB_OK;
	}
	else
	{
		if (qleft) 
		{
			cdflib_unabletoinvert("cdflib_chi2inv", 0., "x");
			status = CDFLIB_ERROR;
		}
		else
		{
			cdflib_unabletoinvert("cdflib_chi2inv", large, "x");
			status = CDFLIB_ERROR;
		}
	}
	return status;
}
