/*
*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 1994 - Barry W. Brown, James Lovato, Kathy Russell (DCDFLIB)
*
*/

/* cdff.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include <math.h>

#include "cdflib.h"
#include "cdflib_private.h" 

/* ********************************************************************** */

// F Distribution

/*                              Method */

/*     Formula   26.6.2   of   Abramowitz   and   Stegun,  Handbook  of */
/*     Mathematical  Functions (1966) is used to reduce the computation */
/*     of the  cumulative  distribution function for the  F  variate to */
/*     that of an incomplete beta. */

/*     Computation of other parameters involve a seach for a value that */
/*     produces  the desired  value  of P.   The search relies  on  the */
/*     monotinicity of P with the other parameter. */

/*                              WARNING */

/*     The value of the  cumulative  F distribution is  not necessarily */
/*     monotone in  either degrees of freedom.  There  thus may  be two */
/*     values  that  provide a given CDF  value.   This routine assumes */
/*     monotonicity and will find an arbitrary one of the two values. */

/* ********************************************************************** */

int cdflib_fCheckParams(char * fname, double v1, double v2)
{
	int status;
	/*     V1 > 0 */
	status=cdflib_checkgreaterthan(fname, v1, "v1", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/*     V2 > 0 */
	status=cdflib_checkgreaterthan(fname, v2, "v2", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}
int cdflib_fCheckX(char * fname, double x)
{
	int status;
	/*     X >=0 */
	status=cdflib_checkgreqthan(fname, x, "x", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}

int cdflib_fcdf(double x, double v1, double v2, int lowertail, double *p)
{

	int status;
	double q;

	// Check X
	status=cdflib_fCheckX("cdflib_fcdf",x);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	// Check V1, V2
	status=cdflib_fCheckParams("cdflib_fcdf",v1, v2);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	if (cdflib_isnan(x) || cdflib_isnan(v1) || cdflib_isnan(v2))
	{
		*p = x+v1+v2;
		status = CDFLIB_OK;
		return status;
	}
	cdflib_cumf(x, v1, v2, p, &q);
	status = CDFLIB_OK;
	if (lowertail==CDFLIB_UPPERTAIL)
	{
		*p=q;
	}
	return status;
}
int cdflib_finv(double p, double v1, double v2, int lowertail, double *x)
{

	int status;
	/* Local variables */
	static double fx;
	static long int qhi;
	static double cum, ccum;
	static long int qleft;
	static long int qporq;

	double large = cdflib_doubleLarge();
	double rtol = cdflib_doubleEps();
	double infinite = cdflib_infinite();
	double q;

	/* Table of constant values */
	static double c_b24 = 0.;
	static double c_b26 = .5;
	static double c_b28 = 5.;
	static double c_b29 = 1e-50;

	// P in [0,1]
	status=cdflib_checkp("cdflib_finv", p, "p");
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Check V1, V2
	status=cdflib_fCheckParams("cdflib_finv", v1, v2);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Compute the couple (p,q) from (p,lowertail)
	if (lowertail==CDFLIB_LOWERTAIL)
	{
		q=1-p;
	}
	else
	{
		q=p;
		p=1-q;
	}
	if (q==0)
	{
		*x = infinite;
		status = CDFLIB_OK;
		return status;
	}
	if (cdflib_isnan(p) || cdflib_isnan(q) || cdflib_isnan(v1) || cdflib_isnan(v2))
	{
		*x = p+q+v1+v2;
		status = CDFLIB_OK;
		return status;
	}
	/*     Calculating X */
	*x = 5.;
	cdflib_dstinv(&c_b24, &large, &c_b26, &c_b26, &c_b28, &c_b29, &rtol);
	status = 0;
	cdflib_dinvr(&status, x, &fx, &qleft, &qhi);
	while (status == 1)
	{
		cdflib_cumf(*x, v1, v2, &cum, &ccum);
		fx = cdflib_computefx(p, q, cum, ccum);
		cdflib_dinvr(&status, x, &fx, &qleft, &qhi);
	}
	if (status == 0) 
	{
		status = CDFLIB_OK;
	}
	else
	{
		if (qleft) 
		{
			cdflib_unabletoinvert("cdflib_finv", 0., "x");
			status = CDFLIB_ERROR;
		}
		else
		{
			cdflib_unabletoinvert("cdflib_finv", large, "x");
			status = CDFLIB_ERROR;
		}
	}
	return status;
}
