/*
*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 1994 - Barry W. Brown, James Lovato, Kathy Russell (DCDFLIB)
*
*/

/* cdfgam.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include <math.h>
#include <stdio.h>

#include "cdflib.h"
#include "cdflib_private.h" 

/* ********************************************************************** */

// Gamma Distribution

/*                              Method */

/*     Cumulative distribution function (P) is calculated directly by */
/*     the code associated with: */

/*     DiDinato, A. R. and Morris, A. H. Computation of the  incomplete */
/*     gamma function  ratios  and their  inverse.   ACM  Trans.  Math. */
/*     Softw. 12 (1986), 377-393. */

/*     Computation of other parameters involve a seach for a value that */
/*     produces  the desired  value  of P.   The search relies  on  the */
/*     monotinicity of P with the other parameter. */

/*                              Note */

/*     The gamma density is proportional to */
/*       X**(A - 1) * EXP(- X / B) */

/* ********************************************************************** */

int cdflib_gamCheckX(char * fname, double x)
{
	int status;

	/*     X >=0 */
	status=cdflib_checkgreqthan(fname, x, "x", 0);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}
int cdflib_gamCheckParams(char * fname, double a, double b)
{
	int status;

	/*     A > 0 */
	status=cdflib_checkgreaterthan(fname, a, "a", 0);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/*     B > 0 */
	status=cdflib_checkgreaterthan(fname, b, "b", 0);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}

int cdflib_gaminv(double p, double a, double b, int lowertail, double *x)
{

	int status;

	/* Local variables */
	double xx;
	int ierr;
	double q;

	double infinite = cdflib_infinite();

	// P in [0,1]
	status=cdflib_checkp("cdflib_gaminv", p, "p");
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	// Check A, B
	status=cdflib_gamCheckParams("cdflib_gaminv", a, b);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Compute the couple (p,q) from (p,lowertail)
	if (lowertail==CDFLIB_LOWERTAIL)
	{
		q=1-p;
	}
	else
	{
		q=p;
		p=1-q;
	}

	/*     Computing X */
	if (q==0)
	{
		*x = infinite;
		status = CDFLIB_OK;
		return status;
	}
	if (cdflib_isnan(p) || cdflib_isnan(q) || cdflib_isnan(a) || cdflib_isnan(b))
	{
		*x = p+q+a+b;
		status = CDFLIB_OK;
		return status;
	}
	cdflib_gaminvfun(a, &xx, -1, p, q, &ierr);
	if (ierr < 0) 
	{
		char buffer [1024];
		sprintf (buffer, "%s: Unable to evaluate Gamma CDF at x=""%s"".\n","cdfgam","x");
		cdflib_messageprint(buffer);
		status = CDFLIB_ERROR;
	} 
	else 
	{
		*x = xx * b;
		status = CDFLIB_OK;
	}
	return status;
}

int cdflib_gamcdf(double x, double a, double b, int lowertail, double *p)
{

	int status;

	/* Local variables */
	static double xx;
	static int ierr;
	static double xrate;

	double infinite = cdflib_infinite();
	double q;

	// Check X
	status=cdflib_gamCheckX("cdflib_gamcdf",x);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	// Check A, B
	status=cdflib_gamCheckParams("cdflib_gamcdf",a, b);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/*     Calculating P and Q */
	if (x==infinite)
	{
		if (lowertail==CDFLIB_LOWERTAIL)
		{
			*p=1.;
		}
		else
		{
			*p=0.;
		}
		status = CDFLIB_OK;
		return status;
	}
	if (cdflib_isnan(x) || cdflib_isnan(a) || cdflib_isnan(b))
	{
		*p = x+a+b;
		status = CDFLIB_OK;
		return status;
	}
	xrate = x / b;
	cdflib_cumgam(xrate, a, p, &q);
	if (*p > 1.5)
	{
		char buffer [1024];
		sprintf (buffer, "%s: Unable to evaluate Gamma CDF at x=""%s"".\n","cdfgam","x");
		cdflib_messageprint(buffer);
		status = CDFLIB_ERROR;
		return status;
	}
	if (lowertail==CDFLIB_UPPERTAIL)
	{
		*p=q;
	}
	status = CDFLIB_OK;
	return status;
}
