
/*
 * Copyright (C) 2012 - Michael Baudin
 *
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 *
 */

#include <string.h>
#include <stdio.h>
#include <math.h>

#include "cdflib.h"
#include "cdflib_private.h" 

#ifdef _MSC_VER
#include <float.h>
#endif

// Parameters used in the cdf*.c functions

double cdflib_doubleEps(void)
{
	double ret_val;
	double eps;
	eps = cdflib_spmpar(1);
	ret_val = 2*eps;
	return ret_val;

}

double cdflib_doubleTiny(void)
{
	double ret_val;
	ret_val = cdflib_spmpar(2);
	return ret_val;
}

double cdflib_doubleLarge(void)
{
	double ret_val;
	ret_val = cdflib_spmpar(3);
	return ret_val;
}

double cdflib_infinite(void)
{
	double ret_val;
	double one;
	one=1.;
	ret_val = 1./(one-one);
	return ret_val;
}

double cdflib_isnan(double x)
{
	double r;
#ifdef _MSC_VER 
	r=_isnan(x);
#else 
	r=isnan(x);
#endif 
	return r;
}

// Initialize the message function to NULL.
void (* cdflib_messagefunction)(char * message) = NULL;

// Prints a message.
void cdflib_messageprint ( char * message ) 
{
	if (cdflib_messagefunction==NULL) {
		printf("%s\n",message);
	}
	else 
	{
		cdflib_messagefunction(message);
	}
}

// Configures the message function
void cdflib_messageSetFunction ( void (* f)(char * message) ) 
{
	cdflib_messagefunction = f;
}

// Checks that value>=minparam and value<=maxparam (int).
int cdflib_checkrangeint(char * fname, int param, 
	char * paramname, int minparam, int maxparam) 
{
	char buffer [1024];
	if (param < minparam) 
	{
		sprintf (buffer, CDFLIB_MSGINTGREQ,fname,paramname,minparam);
		cdflib_messageprint(buffer);
		return ( CDFLIB_ERROR );
	} 
	else if (param > maxparam) 
	{
		sprintf (buffer, CDFLIB_MSGINTLOWEQ,fname,paramname,maxparam);
		cdflib_messageprint(buffer);
		return ( CDFLIB_ERROR );
	}
	return ( CDFLIB_OK );
}

// Checks that value>=minparam and value<=maxparam (double).
int cdflib_checkrangedouble(char * fname, double param, 
	char * paramname, double minparam, double maxparam) 
{
	char buffer [1024];
	if (param < minparam) 
	{
		sprintf (buffer, CDFLIB_MSGFLOATGREQ,fname,paramname,minparam);
		cdflib_messageprint(buffer);
		return ( CDFLIB_ERROR );
	} 
	else if (param > maxparam) 
	{
		sprintf (buffer, CDFLIB_MSGFLOATLOWEQ,fname,paramname,maxparam);
		cdflib_messageprint(buffer);
		return ( CDFLIB_ERROR );
	}
	return ( CDFLIB_OK );
}

// Checks the value of p in [0,1]
int cdflib_checkp(char * fname, double p, char * pname) 
{
	int status;
	// Check P
	status = cdflib_checkrangedouble(fname, p, pname, 0., 1.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return ( CDFLIB_OK );
}

// Checks the values of p and q
int cdflib_checkpq(char * fname, double p, char * pname, double q, char * qname) 
{
	int status;
	char pqname [1024];
	// Check P
	status = cdflib_checkrangedouble(fname, p, pname, 0., 1.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	// Check Q
	status = cdflib_checkrangedouble(fname, q, qname, 0., 1.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	// Check P+Q
	sprintf (pqname, "%s+%s",pname,qname);
	status = cdflib_checksumtoone(fname, p, q, pqname);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}

// Checks that x+y==1.
// The formal name for x+y is paramname.
int cdflib_checksumtoone(char * fname, double x, double y, char * paramname) 
{
	double xyminus1;
	double xy;	
	double epsilon;
	char buffer [1024];

	//
	xy = x + y;
	xyminus1 = xy - .5 - .5;
	epsilon = cdflib_spmpar(1);
	if (fabs(xyminus1) > epsilon * 3.) 
	{
		sprintf (buffer, "%s: Wrong value for input arguments ""%s"": %s.\n",fname,paramname,"not equal to 1");
		cdflib_messageprint(buffer);
		return ( CDFLIB_ERROR );
	}
	if (xy < 0.) 
	{
		sprintf (buffer, "%s: Wrong value for input arguments ""%s"": %s.\n",fname,paramname,"< 0");
		cdflib_messageprint(buffer);
		return ( CDFLIB_ERROR );
	}
	return (CDFLIB_OK);
}

// Checks that value>minparam.
int cdflib_checkgreaterthan(char * fname, double param, 
	char * paramname, double minparam) 
{
	char buffer [1024];
	if (param <= minparam) 
	{
		sprintf (buffer, CDFLIB_MSGFLOATGREATER,fname,paramname,minparam);
		cdflib_messageprint(buffer);
		return ( CDFLIB_ERROR );
	} 
	return ( CDFLIB_OK );
}

// Checks that value>=minparam.
int cdflib_checkgreqthan(char * fname, double param, 
	char * paramname, double minparam) 
{
	char buffer [1024];
	if (param < minparam) 
	{
		sprintf (buffer, CDFLIB_MSGFLOATGREQ,fname,paramname,minparam);
		cdflib_messageprint(buffer);
		return ( CDFLIB_ERROR );
	} 
	return ( CDFLIB_OK );
}

// Checks that value<=maxparam.
int cdflib_checkloweqthan(char * fname, double param, 
	char * paramname, double maxparam) 
{
	char buffer [1024];
	if (param > maxparam) 
	{
		sprintf (buffer, CDFLIB_MSGFLOATLOWEQ,fname,paramname,maxparam);
		cdflib_messageprint(buffer);
		return ( CDFLIB_ERROR );
	} 
	return ( CDFLIB_OK );
}
int cdflib_checkintloweqthan(char * fname, int param, 
	char * paramname, int maxparam) 
{
	char buffer [1024];
	if (param > maxparam) 
	{
		sprintf (buffer, CDFLIB_MSGINTLOWEQ,fname,paramname,maxparam);
		cdflib_messageprint(buffer);
		return ( CDFLIB_ERROR );
	} 
	return ( CDFLIB_OK );
}

// Computes the zero function for inverse CDF computation.
double cdflib_computefx(double p, double q, double cum, double ccum) 
{
	double fx;
	if (p <= q) 
	{
		fx = cum - p;
	}
	else
	{
		fx = ccum - q;
	}
	return fx;
}

// Print a "Unable to invert message"
void cdflib_unabletoinvert(char * fname, double bound, char * paramname) 
{
	char buffer [1024];
	sprintf (buffer, "%s: Unable to compute ""%s"". Bound reached=%e.\n",fname,paramname,bound);
	cdflib_messageprint(buffer);
	return;
}
// Checks that value>minparam.
int cdflib_checkintgreaterthan(char * fname, int param, 
	char * paramname, int minparam) 
{
	char buffer [1024];
	if (param <= minparam) 
	{
		sprintf (buffer, CDFLIB_MSGINTGREATER,fname,paramname,minparam);
		cdflib_messageprint(buffer);
		return ( CDFLIB_ERROR );
	} 
	return ( CDFLIB_OK );
}

// Checks that value>=minparam.
int cdflib_checkintgreqthan(char * fname, int param, 
	char * paramname, int minparam) 
{
	char buffer [1024];
	if (param < minparam) 
	{
		sprintf (buffer, CDFLIB_MSGINTGREQ,fname,paramname,minparam);
		cdflib_messageprint(buffer);
		return ( CDFLIB_ERROR );
	} 
	return ( CDFLIB_OK );
}
// Check the tail option of a function
int cdflib_checklowertail(char * fname, int lowertail) 
{
	char buffer [1024];
	if (lowertail!=CDFLIB_UPPERTAIL && lowertail!=CDFLIB_LOWERTAIL)
	{
		sprintf (buffer, CDFLIB_MSGGENERIC,fname,"lowertail","0 or 1");
		cdflib_messageprint(buffer);
		return ( CDFLIB_ERROR );
	}
	return ( CDFLIB_OK );
}

// Check that a double has an integer value.
int cdflib_checkIntValue(char * fname, double x, char * paramname)
{
	double t;
	char buffer [1024];
	t = floor(x);
	if (x!=t)
	{
		sprintf (buffer, CDFLIB_MSGFLOATVALUEINT,fname,"x");
		cdflib_messageprint(buffer);
		return ( CDFLIB_ERROR );
	} 
	return ( CDFLIB_OK );
}

