/*
*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 1994 - Barry W. Brown, James Lovato, Kathy Russell (DCDFLIB)
*
*/

/* cdfnbn.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include <math.h>

#include "cdflib.h"
#include "cdflib_private.h" 

/* ********************************************************************** */

/*               Cumulative Distribution Function */
/*               Negative BiNomial distribution */


/*                              Function */


/*     Calculates any one parameter of the negative binomial */
/*     distribution given values for the others. */

/*     The  cumulative  negative   binomial  distribution  returns  the */
/*     probability that there  will be  F or fewer failures before  the */
/*     XNth success in binomial trials each of which has probability of */
/*     success PR. */

/*     The individual term of the negative binomial is the probability of */
/*     X failures before N successes and is */
/*          Choose( X, N+X-1 ) * PR^(N) * (1-PR)^X */

/*                              Arguments */

/*     WHICH --> Integer indicating which of the next four argument */
/*               values is to be calculated from the others. */
/*               Legal range: 1,2 */
/*               iwhich = 1 : Calculate P and Q from X,N,PR and OMPR */
/*               iwhich = 2 : Calculate X from P,Q,N,PR and OMPR */


/*     P <--> The cumulation from 0 to X of the  negative */
/*            binomial distribution. */
/*            Input range: [0,1]. */


/*     Q <--> 1-P. */
/*            Input range: (0, 1]. */
/*            P + Q = 1.0. */


/*     X <--> The upper limit of cumulation of the binomial distribution. */
/*            There are F or fewer failures before the XNth success. */
/*            Input range: [0, +infinity). */
/*            Search range: [0, large] */


/*     N  <--> The number of successes. */
/*              Input range: [0, +infinity). */
/*              Search range: [0, large] */


/*     PR  <--> The probability of success in each binomial trial. */
/*              Input range: [0,1]. */
/*              Search range: [0,1]. */


/*     OMPR  <--> 1-PR */
/*              Input range: [0,1]. */
/*              Search range: [0,1] */
/*              PR + OMPR = 1.0 */


/*     STATUS <-- CDFLIB_OK if calculation completed correctly */
/*                CDFLIB_ERROR if not */

/*                              Method */


/*     Formula   26.5.26   of   Abramowitz  and  Stegun,  Handbook   of */
/*     Mathematical Functions (1966) is used  to  reduce calculation of */
/*     the cumulative distribution  function to that of  an  incomplete */
/*     beta. */

/*     Computation of other parameters involve a seach for a value that */
/*     produces  the desired  value  of P.   The search relies  on  the */
/*     monotinicity of P with the other parameter. */


/* ********************************************************************** */

/* Table of constant values */

static int c__1 = 1;
static double c_b35 = 0.;
static double c_b37 = .5;
static double c_b39 = 5.;
static double c_b40 = 1e-50;
static double c_b64 = 1.;

/* Subroutine */ int cdflib_cdfnbn(int which, double *p, double *q, 
	double *x, double n, double pr, double ompr)
{

	int status;

	/* Local variables */
	static double fx, pq;
	static long int qhi;
	static double cum, xhi, xlo, ccum;
	static long int qleft;
	static long int qporq;
	static double prompr;

	double large = cdflib_doubleLarge();
	double rtol = cdflib_doubleEps();


	/*     Check arguments */

	status=cdflib_checkrangeint("cdfnbn", which, "which", 1, 2);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/* P, Q, P + Q */
	if (which==2)
	{
		status=cdflib_checkpq("cdfnbn", *p, "p", *q, "q");
		if (status!=CDFLIB_OK)
		{
			return status;
		}
	}

	// Check X
	if (which==1)
	{
		status=cdflib_nbnCheckX("cdfnbn", *x);
		if (status!=CDFLIB_OK)
		{
			return status;
		}
	}

	// Check N, PR, OMPR
	status=cdflib_nbnCheckParams("cdfnbn", n, pr, ompr);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/*     Calculate ANSWERS */
	if (which==1) 
	{
		/*     Calculating P and Q */
		if (cdflib_isnan(*x) || cdflib_isnan(n) || cdflib_isnan(pr) || cdflib_isnan(ompr))
		{
			*p = *x+n+pr+ompr;
			*q = *x+n+pr+ompr;
			status = CDFLIB_OK;
			return status;
		}
		cdflib_cumnbn(*x, n, pr, ompr, p, q);
		status = CDFLIB_OK;
	} 
	else if (which==2) 
	{
		/*     Calculating X */
		if (cdflib_isnan(*p) || cdflib_isnan(*q) || cdflib_isnan(n) || cdflib_isnan(pr) || cdflib_isnan(ompr))
		{
			*x = *p+*q+n+pr+ompr;
			status = CDFLIB_OK;
			return status;
		}
		*x = 5.;
		cdflib_dstinv(&c_b35, &large, &c_b37, &c_b37, &c_b39, &c_b40, &rtol);
		status = 0;
		cdflib_dinvr(&status, x, &fx, &qleft, &qhi);
		while (status == 1)
		{
			cdflib_cumnbn(*x, n, pr, ompr, &cum, &ccum);
			fx = cdflib_computefx(*p, *q, cum, ccum);
			cdflib_dinvr(&status, x, &fx, &qleft, &qhi);
		}
		if (status == 0) 
		{
			status = CDFLIB_OK;
		}
		else
		{
			if (qleft) 
			{
				cdflib_unabletoinvert("cdfnbn", 0., "x");
				status = CDFLIB_ERROR;
			}
			else
			{
				cdflib_unabletoinvert("cdfnbn", large, "x");
				status = CDFLIB_ERROR;
			}
		}
	}
	return status;
}


int cdflib_nbnCheckX(char * fname, double x)
{
	int status;
	/*     X >= 0 */
	status=cdflib_checkgreqthan(fname, x, "x", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}
int cdflib_nbnCheckParams(char * fname, double n, double pr, double ompr)
{
	int status;
	/*     N >= 0 */
	status=cdflib_checkgreqthan(fname, n, "n", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/*     PR, OMPR, PR + OMPR */
	status=cdflib_checkpq(fname, pr, "pr", ompr, "ompr");
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}
