/*
*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 1994 - Barry W. Brown, James Lovato, Kathy Russell (DCDFLIB)
*
*/

/* cdfnor.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include <math.h>

#include "cdflib.h"
#include "cdflib_private.h" 

/* ********************************************************************** */

// Normal Distribution

/*                              Method */

/*     A slightly modified version of ANORM from */

/*     Cody, W.D. (1993). "ALGORITHM 715: SPECFUN - A Portabel FORTRAN */
/*     Package of Special Function Routines and Test Drivers" */
/*     acm Transactions on Mathematical Software. 19, 22-32. */

/*     is used to calulate the  cumulative standard normal distribution. */

/*     The rational functions from pages  90-95  of Kennedy and Gentle, */
/*     Statistical  Computing,  Marcel  Dekker, NY,  1980 are  used  as */
/*     starting values to Newton's Iterations which compute the inverse */
/*     standard normal.  Therefore no  searches  are necessary for  any */
/*     parameter. */

/*     For X < -15, the asymptotic expansion for the normal is used  as */
/*     the starting value in finding the inverse standard normal. */
/*     This is formula 26.2.12 of Abramowitz and Stegun. */

/*                              Note */

/*      The normal density is proportional to */
/*      exp( - 0.5 * (( X - MU)/SIGMA)**2) */

/* ********************************************************************** */


int cdflib_norCheckParams(char * fname, double mu, double sigma)
{
	int status;
	// Nothing to do for mu

	/*     SIGMA > 0 */
	status=cdflib_checkgreaterthan(fname, sigma, "sigma", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}
int cdflib_norCheckX(char * fname, double x)
{
	// Nothing to do...
	return CDFLIB_OK;
}
int cdflib_normcdf(double x, double mu, double sigma, int lowertail, double *p)
{
	int status;
	static double a;
	double infinite = cdflib_infinite();
	double q;
	// Check X
	status=cdflib_norCheckX("cdflib_normcdf", x);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	// Check MU, SIGMA
	status=cdflib_norCheckParams("cdflib_normcdf", mu, sigma);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	/*     Computing P and Q */
	if (x==-infinite)
	{
		if (lowertail==CDFLIB_LOWERTAIL)
		{
			*p=0.;
		}
		else
		{
			*p=1.;
		}
		status = CDFLIB_OK;
		return status;
	}
	if (x==infinite)
	{
		if (lowertail==CDFLIB_LOWERTAIL)
		{
			*p=1.;
		}
		else
		{
			*p=0.;
		}
		status = CDFLIB_OK;
		return status;
	}
	if (cdflib_isnan(x) || cdflib_isnan(mu) || cdflib_isnan(sigma))
	{
		*p = x+mu+sigma;
		status = CDFLIB_OK;
		return status;
	}
	a = (x - mu) / sigma;
	cdflib_cumnor(a, p, &q);
	if (lowertail==CDFLIB_UPPERTAIL)
	{
		*p=q;
	}
	status = CDFLIB_OK;
	return status;
}
int cdflib_norminv(double p, double mu, double sigma, int lowertail, double *x)
{
	int status;
	double infinite = cdflib_infinite();
	double q;
	double z;
	// P in [0,1]
	status=cdflib_checkp("cdflib_norminv", p, "p");
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Check MU, SIGMA
	status=cdflib_norCheckParams("cdflib_norminv", mu, sigma);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Compute the couple (p,q) from (p,lowertail)
	if (lowertail==CDFLIB_LOWERTAIL)
	{
		q=1-p;
	}
	else
	{
		q=p;
		p=1-q;
	}
	if (q==0)
	{
		*x = infinite;
		status = CDFLIB_OK;
		return status;
	}
	if (p==0)
	{
		*x = -infinite;
		status = CDFLIB_OK;
		return status;
	}
	if (cdflib_isnan(p) || cdflib_isnan(mu) || cdflib_isnan(sigma))
	{
		*x = p+mu+sigma;
		status = CDFLIB_OK;
		return status;
	}
	z = cdflib_dinvnr(p, q);
	*x = sigma * z + mu;
	status = CDFLIB_OK;
	return status;
}
