/*
*
* Copyright (C) 2012 - Michael Baudin
* Copyright (C) 1994 - Barry W. Brown, James Lovato, Kathy Russell (DCDFLIB)
*
*/

/* cdfpoi.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include <math.h>

#include "cdflib.h"
#include "cdflib_private.h" 

/* ********************************************************************** */

// Poisson Distribution

/*                              Method */

/*     Formula   26.4.21  of   Abramowitz  and   Stegun,   Handbook  of */
/*     Mathematical Functions (1966) is used  to reduce the computation */
/*     of  the cumulative distribution function to that  of computing a */
/*     chi-square, hence an incomplete gamma function. */

/*     Cumulative  distribution function  (P) is  calculated  directly. */
/*     Computation of other parameters involve a seach for a value that */
/*     produces  the desired value of  P.   The  search relies  on  the */
/*     monotinicity of P with the other parameter. */

/* ********************************************************************** */

int cdflib_poisscdf(double x, double lambda, int lowertail, double *p)
{
	int status;
	double q;
	status=cdflib_poissCheckX("cdflib_poisscdf",x);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Check LAMBDA
	status=cdflib_poissCheckParams("cdflib_poisscdf",lambda);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Calculating P
	if (cdflib_isnan(x) || cdflib_isnan(lambda))
	{
		*p = x+lambda;
		status = CDFLIB_OK;
		return status;
	}
	cdflib_cumpoi(x, lambda, p, &q);
	if (lowertail==CDFLIB_UPPERTAIL)
	{
		*p=q;
	}
	status = CDFLIB_OK;
	return status;
}

int cdflib_poissinv(double p, double lambda, int lowertail, double *x)
{
	int status;
	/* Local variables */
	static double fx, pq;
	static long int qhi;
	static double cum, ccum;
	static long int qleft;

	double large = cdflib_doubleLarge();
	double rtol = cdflib_doubleEps();
	double infinite = cdflib_infinite();

	/* Table of constant values */
	static int c__1 = 1;
	static double c_b22 = 0.;
	static double c_b24 = .5;
	static double c_b26 = 5.;
	static double c_b27 = 1e-50;

	double q;

	// P in [0,1]
	status=cdflib_checkp("cdflib_poissinv", p, "p");
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Check LAMBDA
	status=cdflib_poissCheckParams("cdflib_poissinv",lambda);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Compute the couple (p,q) from (p,lowertail)
	if (lowertail==CDFLIB_LOWERTAIL)
	{
		q=1-p;
	}
	else
	{
		q=p;
		p=1-q;
	}
	// Calculating X
	if (cdflib_isnan(p) || cdflib_isnan(lambda))
	{
		*x = p+lambda;
		status = CDFLIB_OK;
		return status;
	}
	//
	// Fix for small values of p
	*x = 0;
	cdflib_cumpoi(*x, lambda, &cum, &ccum);
	if (p<=cum)
	{
		// The value of p is too small to be 
		// attained by the CDF.
		// The solution is X=0
		status = CDFLIB_OK;
		return status;
	}
	//
	// M. Baudin
	// Fix for small values of p
	if (q==0.)
	{
		// p=1 and q=0.
		// The solution is X=INF.
		*x=infinite;
		status = CDFLIB_OK;
		return status;
	}
	*x = 5.;
	cdflib_dstinv(&c_b22, &large, &c_b24, &c_b24, &c_b26, &c_b27, &rtol);
	status = 0;
	cdflib_dinvr(&status, x, &fx, &qleft, &qhi);
	while (status == 1)
	{
		cdflib_cumpoi(*x, lambda, &cum, &ccum);
		fx = cdflib_computefx(p, q, cum, ccum);
		cdflib_dinvr(&status, x, &fx, &qleft, &qhi);
	}
	if (status == 0) 
	{
		*x=ceil(*x);
		status = CDFLIB_OK;
	}
	else
	{
		if (qleft) 
		{
			cdflib_unabletoinvert("cdflib_poissinv", 0., "x");
			status = CDFLIB_ERROR;
		} 
		else
		{
			cdflib_unabletoinvert("cdflib_poissinv", large, "x");
			status = CDFLIB_ERROR;
		}
	}
	return status;
}
